use Object
---------------------------------------------------------------------------------------------------
IF EXISTS(SELECT * FROM sysobjects WHERE name = '_ListDiskVolumes' AND type = 'P')
  DROP PROCEDURE _ListDiskVolumes
GO

/*
Lists the disk volumes under the calling object
 */
CREATE PROCEDURE _ListDiskVolumes
  @cid              ClassID,
  @id               ObjID,
  @LocationPattern  NVARCHAR(255) = NULL,
  @NamePattern      NVARCHAR(255) = NULL,
  @MinAlert         INT = 0,
  @MinPriority      INT = 0,
  @OpenNotes        INT = 2
AS
  -- Standard TRANSACTION logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN _ListDiskVolumes
  ELSE
    SAVE TRAN _ListDiskVolumes

  IF @cid = 'LOB'
    SELECT @cid = dst_cid, @id = dst_id FROM link
    WHERE link_type = 'LOBL' AND src_cid = @cid AND src_id = @id AND deleted = 0

  -- CREATE TABLE of all appropriate storage groups under the calling object
  CREATE TABLE #descendent
  (
    cid   ClassID,
    id    ObjID,
    gen   INT
  )
  EXEC asisp_descendents @cid, @id, 'IN (''STRG'')'

  -- SET up additional filtering criteria based ON parameters
  DECLARE @NameExpr NVARCHAR(255), @AlertExpr NVARCHAR(40), @PriorityExpr NVARCHAR(40), @NotesExpr NVARCHAR(40)

  IF @NamePattern IS NOT NULL
    SET @NameExpr = ' AND name LIKE ''' + @NamePattern + ''''
  ELSE
    SET @NameExpr = ''

  IF @MinAlert IS NOT NULL AND @MinAlert <> 0
    SET @AlertExpr = ' AND _AlertStateID >= ' + CONVERT(NVARCHAR(9), @MinAlert)
  ELSE
    SET @AlertExpr = ''

  IF @MinPriority IS NOT NULL AND @MinPriority <> 0
    SET @PriorityExpr = ' AND _PriorityID <= ' + CONVERT(NVARCHAR(9), @MinPriority) -- yes, priorities work IN reverse!
  ELSE
    SET @PriorityExpr = ''

  IF @OpenNotes IS NOT NULL AND @OpenNotes <> 2
    SET @NotesExpr = ' AND _HasOwnershipNote = ' + CONVERT(NCHAR(1), @OpenNotes)
  ELSE
    SET @NotesExpr = ''

  -- Populate TABLE with all disk volumes under the storage groups that match the criteria
  CREATE TABLE #obj
  (
    src_cid            ClassID,
    src_id             ObjID,
    cid                ClassID,
    id                 ObjID,
    Location           NVARCHAR(255) NULL,
    name               ObjName NULL,
    _AlertStateID      INT NULL,
    _PriorityID        INT NULL,
    _CurrentStateID    INT NULL,
    _DesiredStateID    INT NULL,
    _HasOwnershipNote  TINYINT NULL,
    Description        NVARCHAR(255) NULL
  )
  EXEC("
  INSERT #obj
    SELECT
      d.cid,
      d.id,
      'DKVL',
      obj.id,
      obj.name,
      obj.name,
      obj._AlertStateID,
      obj._PriorityID,
      obj._CurrentStateID,
      obj._DesiredStateID,
      obj._HasOwnershipNote,
      obj._Desc
    FROM #descendent AS d
      JOIN DiskVolume_V AS obj ON obj._ParentCID = d.cid AND obj._ParentID = d.id AND obj.deleted = 0
    WHERE 1=1"
      + @NameExpr
      + @AlertExpr
      + @PriorityExpr
      + @NotesExpr
  )

  DECLARE @rc INT, @src_cid ClassID, @loop INT, @cname NVARCHAR(24)
  SELECT @rc = COUNT(*) FROM #obj
  SET @cname = ''
  SET @loop = 0
  WHILE @rc > 0 AND @loop < 10
  BEGIN
    DECLARE srcc INSENSITIVE CURSOR FOR SELECT DISTINCT src_cid, cname FROM #obj JOIN obj_class AS c ON c.cid = src_cid
    OPEN srcc
    FETCH NEXT FROM srcc INTO @src_cid, @cname
    WHILE @@FETCH_STATUS <> -1
    BEGIN
      EXEC("
      UPDATE obj SET Location = CASE obj.src_cid
                                  WHEN 'STGF' THEN 'Storage Group Folder' + '/' + Location
                                  ELSE parent.name + '/' + Location
                                END,
                     src_cid = c.src_cid,
                     src_id = c.src_id
      FROM #obj AS obj
      JOIN containment AS c ON c.dst_cid = obj.src_cid AND c.dst_id = obj.src_id
      JOIN " + @cname + "_C AS parent ON parent.id = obj.src_id AND obj.src_cid = c.dst_cid
      WHERE obj.src_cid = '" + @src_cid + "' AND obj.src_cid <> 'BUSC'
      ")
      FETCH NEXT FROM srcc INTO @src_cid, @cname
    END
    DEALLOCATE srcc
    SELECT @rc = COUNT(*) FROM #obj WHERE src_cid <> 'ENT'
    SET @loop = @loop + 1
  END

  -- Filter out paths that don't match parameter, IF specified
  IF @LocationPattern IS NOT NULL
    DELETE #obj WHERE Location NOT LIKE @LocationPattern

  -- obtain the localized strings
  DECLARE @local_string NVARCHAR(255)
  CREATE TABLE #localized_strings
  (
    key_name         sysname,
    english_string   VARCHAR(128),
    local_string     NVARCHAR(255) NULL DEFAULT NULL
  )
  INSERT #localized_strings(key_name, english_string) SELECT '_GetManagedObjects', 'Location'
  INSERT #localized_strings(key_name, english_string) SELECT '_GetManagedObjects', 'Type'
  INSERT #localized_strings(key_name, english_string) SELECT '_GetManagedObjects', 'name'
  INSERT #localized_strings(key_name, english_string) SELECT '_GetManagedObjects', 'Description'
  EXEC asisp_getLocalizedString NULL, NULL, @local_string OUTPUT, '#localized_strings'

  -- store the localized strings in variables
  DECLARE @location_heading NVARCHAR(255), @type_heading NVARCHAR(255), @name_heading NVARCHAR(255), @description_heading NVARCHAR(255)
  SELECT @location_heading=local_string FROM #localized_strings WHERE key_name='_GetManagedObjects' AND english_string='Location'
  SELECT @type_heading=local_string FROM #localized_strings WHERE key_name='_GetManagedObjects' AND english_string='Type'
  SELECT @name_heading=local_string FROM #localized_strings WHERE key_name='_GetManagedObjects' AND english_string='name'
  SELECT @description_heading=local_string FROM #localized_strings WHERE key_name='_GetManagedObjects' AND english_string='Description'

  DROP TABLE #localized_strings

  SET NOCOUNT OFF
  -- RETURN results
  EXEC('
  SELECT
    obj._AlertStateID,
    obj._HasOwnershipNote,
    obj.Location AS N'''+@location_heading+''',
    c.label AS N'''+@type_heading+''',
    name AS N'''+@name_heading+''',
    obj._PriorityID,
    obj._CurrentStateID,
    obj._DesiredStateID,
    Description AS N'''+@description_heading+''',
    obj.cid,
    obj.id
  FROM
    #obj AS obj
      JOIN obj_class AS c ON c.cid = obj.cid
  ORDER BY
    obj.Location
  ')

  -- Finish up TRANSACTION
  IF @tcount = 0
    COMMIT TRAN _ListDiskVolumes

GO

