--
-- Defect 78091 - Appserver should set last processed notification.
--
--  Defines:
--     table ObjectQueues..ProcessedNotificationJ
--     table ObjectQueues..LastProcessedNotificationJ
--     index ObjectQueues..ProcessedNotificationJ_ptime
--     view  Object..ProcessedNotificationJ
--     view  Object..LastProcessedNotificationJ
--     view  Object..NotificationStatusJ
--     proc  Object..asisp_setlastprocessednotifJ
--     proc  Object..asisp_getlastprocessednotifJ
--     trigger ObjectQueues..tI_LastProcessedNotificationJ
--     trigger ObjectQueues..tD_LastProcessedNotificationJ
--     trigger ObjectQueues..tU_LastProcessedNotificationJ
--
--  The Java-based application server uses the asisp_getlastprocessednotifJ
--  view at startup to determine start point in PendingNotifications.  The
--  app server calls asisp_setlastprocessednotifJ after processing notification
--  messages.  The view NotificationStatusJ provides a status view
--  of notifications processed by the Java application server and pending
--  notifications.
--
--
-------------------------------------------------------------------------------------------------------------------------------
PRINT '******************************* In ObjectQueues database *******************************'
GO

USE ObjectQueues
GO
-------------------------------------------------------------------------------------------------------------------------------
-- ProcessedNotificationJ keeps track of the rows from PendingEvent that have been processed by the notification service.
-- As the notification service updates LastProcessedNotificationJ (see below) records are added to
-- ProcessedNotificationJ.
PRINT 'Creating table ProcessedNotificationJ'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'ProcessedNotificationJ' AND type = 'U')
  DROP TABLE ProcessedNotificationJ
GO

CREATE TABLE ProcessedNotificationJ
(
  eventno   INT       CONSTRAINT ProcessedNotificationJ_PK PRIMARY KEY CLUSTERED (eventno),
  ptime     DATETIME
)
GO

IF NOT EXISTS (SELECT * FROM sysindexes WHERE id=object_id('ProcessedNotificationJ') AND name='ProcessedNotificationJ_ptime')
  CREATE INDEX ProcessedNotificationJ_ptime ON ProcessedNotificationJ(ptime)
GO

-------------------------------------------------------------------------------------------------------------------------------
-- LastProcessedNotificationJ
-- Single-row table used by the notification server to keep track of the last row in PendingNotification that has been
-- processed.
PRINT 'Creating table LastProcessedNotificationJ'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'LastProcessedNotificationJ' AND type = 'U')
  DROP TABLE LastProcessedNotificationJ
GO

CREATE TABLE LastProcessedNotificationJ
(
  eventno INT,
  ptime   DATETIME
)
GO

INSERT LastProcessedNotificationJ SELECT 0, getdate()
GO


-------------------------------------------------------------------------------------------------------------------------------
PRINT '******************************* In Object database *******************************'
GO

USE Object
GO

-------------------------------------------------------------------------------------------------------------------------------
-- ProcessedNotificationJ
PRINT 'Creating view ProcessedNotificationJ'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE type='U' AND name='ProcessedNotificationJ')
  DROP TABLE ProcessedNotificationJ
GO
IF EXISTS (SELECT * FROM sysobjects WHERE type='V' AND name='ProcessedNotificationJ')
  DROP VIEW ProcessedNotificationJ
GO

CREATE VIEW ProcessedNotificationJ AS
  SELECT * FROM ObjectQueues..ProcessedNotificationJ
GO

-------------------------------------------------------------------------------------------------------------------------------
-- LastProcessedNotificationJ
PRINT 'Creating view LastProcessedNotificationJ'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE type='U' AND name='LastProcessedNotificationJ')
  DROP TABLE LastProcessedNotificationJ
GO
IF EXISTS (SELECT * FROM sysobjects WHERE type='V' AND name='LastProcessedNotificationJ')
  DROP VIEW LastProcessedNotificationJ
GO

CREATE VIEW LastProcessedNotificationJ AS
  SELECT * FROM ObjectQueues..LastProcessedNotificationJ
GO

-------------------------------------------------------------------------------------------------------------------------------
-- NotificationStatusJ
-- View over PendingNotification and ProcessedNotificationJ that gives an at-a-glance view of the status of all
-- events in a human-friendly way.
PRINT 'Creating view NotificationStatusJ'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'NotificationStatusJ' AND type = 'V')
  DROP VIEW NotificationStatusJ
GO

CREATE VIEW NotificationStatusJ AS
  SELECT pe.eventno, pe.ctime, pr.ptime, pe.type, et.name, pe.fields, pe.detail
  FROM PendingNotification pe, ProcessedNotificationJ pr, EventType et
  WHERE pe.eventno *= pr.eventno AND et.type = pe.type
GO


-------------------------------------------------------------------------------------------------------------------------------
PRINT '******************************* In ObjectQueues database *******************************'
GO

USE ObjectQueues
GO


-------------------------------------------------------------------------------------------------------------------------------
-- tI_LastProcessedNotificationJ
PRINT 'Creating trigger tI_LastProcessedNotificationJ'
GO

CREATE TRIGGER tI_LastProcessedNotificationJ ON LastProcessedNotificationJ FOR INSERT AS
  RAISERROR("Cannot insert into LastProcessedNotificationJ", 11, 1)
  ROLLBACK TRAN
GO


-------------------------------------------------------------------------------------------------------------------------------
-- tD_LastProcessedNotificationJ
PRINT 'Creating trigger tD_LastProcessedNotificationJ'
GO

CREATE TRIGGER tD_LastProcessedNotificationJ ON LastProcessedNotificationJ FOR DELETE AS
  RAISERROR("Cannot delete FROM LastProcessedNotificationJ", 11, 1)
  ROLLBACK TRAN
GO


-------------------------------------------------------------------------------------------------------------------------------
-- tU_LastProcessedNotificationJ
PRINT 'Creating trigger tU_LastProcessedNotificationJ '
GO

CREATE TRIGGER tU_LastProcessedNotificationJ ON LastProcessedNotificationJ FOR UPDATE AS
  INSERT ProcessedNotificationJ(eventno, ptime)  SELECT eventno, ptime FROM inserted
GO


-------------------------------------------------------------------------------------------------------------------------------
PRINT '******************************* In Object database *******************************'
GO

USE Object
GO

-------------------------------------------------------------------------------------------------------------------------------
-- asisp_setlastprocessednotifJ
PRINT 'Creating procedure asisp_setlastprocessednotifJ'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_setlastprocessednotifJ' AND type = 'P')
  DROP PROCEDURE asisp_setlastprocessednotifJ
GO

CREATE PROCEDURE asisp_setlastprocessednotifJ
  @eventno  INT
AS
  UPDATE LastProcessedNotificationJ set eventno = @eventno, ptime = getdate()
GO

-------------------------------------------------------------------------------------------------------------------------------
-- asisp_getlastprocessednotifJ
PRINT 'Creating procedure asisp_getlastprocessednotifJ'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_getlastprocessednotifJ' AND type = 'P')
  DROP PROCEDURE asisp_getlastprocessednotifJ
GO

CREATE PROCEDURE asisp_getlastprocessednotifJ
AS
  SELECT eventno FROM LastProcessedNotificationJ
GO

-------------------------------------------------------------------------------------------------------------------------------
-- asisp_getlastprocessednotifJ
PRINT 'Creating procedure asisp_getlastpendingnotification'
GO

IF EXISTS (SELECT * FROM sysobjects WHERE name = 'asisp_getlastpendingnotification' AND type = 'P')
  DROP PROCEDURE asisp_getlastpendingnotification
GO

CREATE PROCEDURE asisp_getlastpendingnotification
AS
  SELECT eventno=MAX(eventno) FROM PendingNotification
GO

PRINT 'Done.'
GO


