PRINT 'Creating procedure cl_processBulkLinks'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='cl_processBulkLinks')
  DROP PROCEDURE cl_processBulkLinks
GO

CREATE PROCEDURE cl_processBulkLinks
@trid INT,
@instrid NVARCHAR(255)
AS
  -- All the attributes are in #events table
  -- Bulk links are created/deleted when source is created by same instrID
  RAISERROR('cl_processBulkLinks: @trid=%d, @instrid=%s', 0, 1, @trid, @instrid)

  -- table for parsing and getting all link combinations
  CREATE TABLE #contd_links (
    src_id ObjID,
    dst_cid ClassID,
    dst_kn NVARCHAR(64),
    dst_kv NVARCHAR(255)
  )

  -- input table for link rules
  CREATE TABLE #linkEventData ( 
    cid 		ClassID,
    iid	 		ObjID,
    dest_cid 		ClassID NULL,
    linkname 		NVARCHAR(128),
    src_direction 	TINYINT,  
    dest_direction 	TINYINT,
    dest_keyname 	NVARCHAR(32),
    dest_condition 	NVARCHAR(512),
    enabled 		BIT,
    excl 		BIT,
    priority 		INT,
    action 		INT
  )

  -- input table to process links
  CREATE TABLE #linkCandidates ( 
    name               NVARCHAR(128),
    src_direction      TINYINT,    
    src_cid            ClassID,
    src_id             ObjID,
    dst_direction      TINYINT,    
    dst_cid            ClassID,
    dst_id             ObjID,
    rule_id            INT 
  )

  -- table of dst_oids if they exist
  CREATE TABLE #dst_oids (
    oid ObjID
  )
  
  DECLARE @ret INT, @name NVARCHAR(255), @dir NVARCHAR(255), @src_cid ClassID, @cok NVARCHAR(32)
  DECLARE @dst_cid ClassID, @kn NVARCHAR(64), @kv NVARCHAR(255), @evtid INT, @cstr NVARCHAR(32)
  DECLARE @disp VARCHAR(8000), @stat NVARCHAR(8)

  -- Convert link direction from string to integer
  -- tosrc(1), todst(2), 2way(3), none(0)
  UPDATE #events 
    SET value=CASE LOWER(value) 
		WHEN 'totgt' THEN '1' 
		WHEN 'tosrc' THEN '2' 
		WHEN '2way' THEN '3' 
		ELSE '0' 
	      END
    WHERE root='link' and keyword='dir'

  -- populate links in #linkCandidates that have src and tgt specified using instid
  -- use rule_id column to store evtDataId
  -- delete link entries that have more than one targets
  INSERT #linkCandidates SELECT DISTINCT '', 0, '', 0, 0, '', 0, evtid FROM #events 
    WHERE root NOT LIKE 'link:tgt2%'

  UPDATE lc SET lc.name=e.value FROM #events e
    JOIN #linkCandidates lc ON (e.evtid=lc.rule_id)
    WHERE e.root='link' AND e.keyword='type'

  -- get source and target class ids
  UPDATE lc SET lc.src_cid=e.value FROM #events e 
    JOIN #linkCandidates lc ON (e.evtid=lc.rule_id)
    WHERE (e.keyword='class' OR e.keyword='prdver') AND (e.root='link:src1:id1' OR e.root='link:src1:likeid1') 
  UPDATE lc SET lc.dst_cid=e.value FROM #events e 
    JOIN #linkCandidates lc ON (e.evtid=lc.rule_id)
    WHERE (e.keyword='class' OR e.keyword='prdver') AND (e.root='link:tgt1:id1' OR e.root='link:tgt1:likeid1') 

  -- get source and target object ids 
  UPDATE lc SET lc.src_id=clc.oid FROM CL_IDCache clc 
    JOIN #linkCandidates lc ON (lc.src_cid=clc.cid)
    JOIN #events e ON (e.evtid=lc.rule_id AND e.value=clc.value)
    WHERE clc.keyword='CL_InstID' AND clc.deleted=0 AND e.keyword='instid' AND 
 	  (e.root='link:src1:id1' OR e.root='link:src1:likeid1') 
  UPDATE lc SET lc.dst_id=clc.oid FROM CL_IDCache clc 
    JOIN #linkCandidates lc ON (lc.dst_cid=clc.cid)
    JOIN #events e ON (e.evtid=lc.rule_id AND e.value=clc.value)
    WHERE clc.keyword='CL_InstID' AND clc.deleted=0 AND e.keyword='instid' AND
 	  (e.root='link:tgt1:id1' OR e.root='link:tgt1:likeid1') 

  -- get ENT ids if already exists
  UPDATE lc SET lc.dst_id=l.src_id FROM link l
    JOIN #linkCandidates lc ON (l.src_cid=lc.dst_cid AND l.dst_cid=lc.src_cid AND l.dst_id=lc.src_id AND l.link_type=lc.name)
    WHERE l.deleted=0 AND lc.dst_cid='ENT'

  -- delete all links that require more processing (done below)
  DELETE FROM #linkCandidates WHERE name='' OR src_cid='' OR dst_cid='' or src_id=0 or dst_id=0
  SELECT @cstr=COUNT(*) FROM #linkCandidates
  SET @cok = 0

  -- TBSM Links that require extra processing
  DELETE e FROM #events e JOIN #linkCandidates lc ON (e.evtid=lc.rule_id)
  DECLARE blnk_c CURSOR FOR SELECT DISTINCT evtid FROM #events 
  OPEN blnk_c
  FETCH NEXT FROM blnk_c INTO @evtid
  
  WHILE @@FETCH_STATUS <> -1 BEGIN
    SET @disp = ''
    SET @stat = 'OK'
    TRUNCATE TABLE #attrs
    TRUNCATE TABLE #contd_links
    INSERT #attrs SELECT root, keyword, value FROM #events WHERE evtid=@evtid
    INSERT #attrs SELECT 'CommonListener', 'instrId', @instrid

    -- gets all the valid combinations of links that are to be processed
    EXEC @ret = cl_parseLinks @name OUTPUT, @dir OUTPUT, @src_cid OUTPUT, @disp OUTPUT
    IF @ret >= 0 BEGIN
      INSERT #linkEventData SELECT @src_cid, src_id, dst_cid, @name, @dir, 0, 
				   dst_kn, dst_kv, 1, 0, NULL, 1 FROM #contd_links
      SET @cok = @cok + 1
    END
    ELSE SET @stat = 'ERROR'

    EXEC cl_logStatus @trid, @evtid, @disp, @stat
    FETCH NEXT FROM blnk_c INTO @evtid
  END
  DEALLOCATE blnk_c
  SET @disp = 'Pre-processed(1) BULK Links, in BULK: ' + @cstr + ', Individually: ' + @cok 
  EXEC cl_logStatus @trid, 0, @disp, 'OK'

  -- Mark rule_id (which actually has evtid) for links that were captured in beginning in bulk
  UPDATE #linkCandidates SET rule_id=0

  DECLARE blnk2_c CURSOR FOR SELECT DISTINCT led.cid, led.linkname, led.src_direction, led.dest_cid, led.dest_keyname, led.dest_condition 
    FROM #linkEventData led, link_type lt WHERE led.linkname=lt.link_type
  OPEN blnk2_c
  FETCH NEXT FROM blnk2_c INTO @src_cid, @name, @dir, @dst_cid, @kn, @kv

  WHILE @@FETCH_STATUS <> -1 BEGIN
    EXEC cl_getLinkDsts @dst_cid, @kn, @kv, 1, @src_cid

    IF EXISTS (SELECT * FROM #dst_oids) BEGIN
      INSERT #linkCandidates SELECT @name, @dir, @src_cid, a.iid, 0, @dst_cid, b.oid, 0
        FROM #linkEventData a CROSS JOIN #dst_oids b        
        WHERE cid=@src_cid AND linkname=@name AND src_direction=@dir AND 
              dest_cid=@dst_cid AND dest_keyname=@kn AND dest_condition=@kv

/*
      -- Dont have to do this as we are not adding any rules in BULK
      -- Delete from #linkEventData, so that its not made link rule
      DELETE FROM #linkEventData 
        WHERE cid=@src_cid AND linkname=@name AND src_direction=@dir AND 
              dest_cid=@dst_cid AND dest_keyname=@kn AND dest_condition=@kv
*/
    END

    FETCH NEXT FROM blnk2_c INTO @src_cid, @name, @dir, @dst_cid, @kn, @kv
  END
  DEALLOCATE blnk2_c
  SET @disp = 'Pre-processed(2) BULK Links - Convert LinkData to LinkCandidates'
  EXEC cl_logStatus @trid, 0, @disp, 'OK'
  TRUNCATE TABLE #linkEventData

  -- contd link processing
  CREATE TABLE #del_links (
    type NVARCHAR(255),
    src_cid ClassID,
    src_id ObjID,
    dst_cid ClassID,
    dst_id ObjID
  )
  -- get existing links for cids in this instrumentation
  INSERT #del_links SELECT l.link_type, l.src_cid, l.src_id, l.dst_cid, l.dst_id
    FROM link l, CL_IDCache c WHERE c.cid=l.dst_cid AND c.oid=l.dst_id AND c.instrID=@instrid 
	 AND l.deleted=0 AND c.deleted=0 AND l.link_type IN (SELECT link_type FROM link_type)
  
  -- get and delete links that exist in TBSM and are not in #linkCandidates
  DELETE e FROM #del_links e, #linkCandidates c WHERE e.type=c.name AND 
    e.src_cid=c.dst_cid AND e.src_id=c.dst_id AND e.dst_cid=c.src_cid AND e.dst_id=c.src_id  

  -- Deleting links in bulk
  DECLARE @dcnt NVARCHAR(32)
  EXEC @dcnt = cl_deleteLinksInBulk
  SET @disp = 'Finished BULK Link Deletion, Deleted Link Count: ' + @dcnt 
  EXEC cl_logStatus @trid, 0, @disp, 'OK'
  TRUNCATE TABLE #del_links

  -- process contd links 
  SET @disp = ''
  IF EXISTS (SELECT * FROM #linkCandidates)  
    EXEC cl_processContd @disp OUTPUT
  SET @disp = 'Finished BULK Link Creation, ' + @disp
  EXEC cl_logStatus @trid, 0, @disp, 'OK'
  
  -- Call link rule processing
--  IF EXISTS (SELECT * FROM #linkEventData)    
--    EXEC cl_processLinksInBulk
