PRINT 'Creating procedure cl_processContd'

IF EXISTS (SELECT * FROM sysobjects WHERE type='P' AND name='cl_processContd')
  DROP PROCEDURE cl_processContd
GO

CREATE PROCEDURE cl_processContd
@disp VARCHAR(8000) OUTPUT
AS
  -- Input in #linkCandidates
  RAISERROR('cl_processContd: @disp OUT', 0, 1)

  -- Standard transaction logic
  DECLARE @tcount INT
  SELECT @tcount = @@TRANCOUNT
  IF @tcount = 0
    BEGIN TRAN cl_processContd
  ELSE
    SAVE TRAN cl_processContd

  CREATE TABLE #linkChanges (
    type NVARCHAR(4),
    src_cid ClassID,
    src_id ObjID,
    dst_cid ClassID,
    dst_id ObjID,
    action_type sysname
  )

  -- Create indices on #linkCandidates
  CREATE CLUSTERED INDEX idx_processContd_1 ON #linkCandidates (name, src_cid, src_id, dst_cid, dst_id)
  CREATE INDEX idx_processContd_2 ON #linkCandidates (src_cid, src_id)

  -- Delete rules after first match
  DELETE lr FROM CL_LinkRules lr 
    JOIN #linkCandidates lc ON (lr.rule_id=lc.rule_id)
    JOIN link_type lt ON (lt.link_type=lr.name)
    WHERE lc.rule_id > 0

  -- Delete links that are not valid to have parent-child relationship
  -- Cannot do this with obj_link_type as its view with data coming from three different tables
  -- and not sure if thats the reason, but outer joins dont work properly in call below
  DELETE a FROM #linkCandidates a LEFT OUTER JOIN derived_obj_link b 
    ON (a.name=b.link_type AND a.src_cid=b.dst_cid AND a.dst_cid=b.src_cid)
    WHERE b.link_type IS NULL
  IF @@ROWCOUNT > 0 SET @disp = @disp + ', Links deleted due to invalid parent-child relation'

  -- Check if a parent already exists for containment links
  -- Cannot make a PHYC link between a child that already has one deleted in link table
  DELETE a FROM #linkCandidates a 
    JOIN containment b ON (a.src_cid=b.dst_cid AND a.src_id=b.dst_id)
    JOIN link_type lt ON (a.name=lt.link_type)
    WHERE lt.flag_containment=1 

  -- Set deleted flag 0 if the link was already deleted before
  /* Link Notification */
  INSERT #linkChanges(type, src_cid, src_id, dst_cid, dst_id, action_type)
    SELECT lc.name, lc.dst_cid, lc.dst_id, lc.src_cid, lc.src_id, 'LinkCreated' FROM #linkCandidates lc
    JOIN link l ON (l.link_type=lc.name AND l.src_cid=lc.dst_cid AND l.src_id=lc.dst_id AND l.dst_cid=lc.src_cid AND l.dst_id=lc.src_id)
    WHERE l.deleted=1

  /* Link Creation */
  UPDATE l SET l.deleted=0 FROM link l 
    JOIN #linkCandidates lc ON (l.link_type=lc.name AND l.src_cid=lc.dst_cid AND l.src_id=lc.dst_id AND l.dst_cid=lc.src_cid AND l.dst_id=lc.src_id)
    WHERE l.deleted=1
  UPDATE c SET c.deleted=0 FROM containment c 
    JOIN #linkCandidates lc ON (c.src_cid=lc.dst_cid AND c.src_id=lc.dst_id AND c.dst_cid=lc.src_cid AND c.dst_id=lc.src_id)
    JOIN link_type lt ON (lt.link_type=lc.name)
    WHERE c.deleted=1 AND lt.flag_containment=1

  -- Check if link already exists
  DELETE a FROM #linkCandidates a, link b 
    WHERE a.name=b.link_type AND a.src_cid=b.dst_cid AND a.src_id=b.dst_id AND
          a.dst_cid=b.src_cid AND a.dst_id=b.src_id AND b.deleted=0

  -- Check for duplicate containment links as they can have ONLY ONE parent
  -- This is in the link list about to be created
  DECLARE @origcnt INT, @nodupcnt INT
  CREATE TABLE #tmp2LC (
    name NVARCHAR(255) NULL, 
    src_cid ClassID NULL,
    src_id ObjID NULL,
    dst_cid ClassID NULL,
    dst_id ObjID NULL
  )

  SELECT name, src_cid, src_id, dst_cid, dst_id INTO #tmp1LC FROM #linkCandidates lc
    JOIN link_type lt ON (lt.link_type=lc.name)
    WHERE lt.flag_containment=1
  SET @origcnt = @@ROWCOUNT
  INSERT #tmp2LC(src_cid, src_id) SELECT DISTINCT src_cid, src_id FROM #tmp1LC
  SET @nodupcnt = @@ROWCOUNT

  IF @origcnt <> @nodupcnt BEGIN
    SET @disp = @disp + ', Duplicate containment links (' + CAST((@origcnt-@nodupcnt) AS NVARCHAR) + ') deleted'
    DELETE lc FROM #linkCandidates lc JOIN #tmp1LC tlc 
      ON (tlc.name=lc.name AND tlc.src_cid=lc.src_cid AND tlc.src_id=lc.src_id AND tlc.dst_cid=lc.dst_cid AND tlc.dst_id=lc.dst_id)

    UPDATE b SET b.name=a.name, b.dst_cid=a.dst_cid, b.dst_id=a.dst_id 
      FROM #tmp1LC a, #tmp2LC b
      WHERE b.src_cid=a.src_cid AND b.src_id=a.src_id

    INSERT #linkCandidates(name, src_cid, src_id, dst_cid, dst_id)
      SELECT name, src_cid, src_id, dst_cid, dst_id FROM #tmp2LC
  END
  TRUNCATE TABLE #tmp1LC
  TRUNCATE TABLE #tmp2LC  
   
  /* Link Creation */
  -- Create new links between (src_cid, src_id) & (dst_cid, dst_id)
  
  DECLARE @name NVARCHAR(255), @src_cid ClassID, @dst_cid ClassID, @table sysname, @sql NVARCHAR(1024), @cnt NVARCHAR(32)
  SET @cnt = 0
  DECLARE lnk_c CURSOR FOR SELECT DISTINCT name, src_cid, dst_cid FROM #linkCandidates
  OPEN lnk_c
  FETCH NEXT FROM lnk_c INTO @name, @src_cid, @dst_cid

  WHILE @@FETCH_STATUS <> -1 BEGIN
    SET @table = RTRIM(@dst_cid) + '_' + RTRIM(@src_cid) + '_' + RTRIM(@name)
    SET @sql = 'INSERT INTO ' + @table + '(src_id, dst_id) SELECT DISTINCT dst_id, src_id FROM #linkCandidates' +
	       '  WHERE name=''' + @name + ''' AND src_cid=''' + @src_cid + ''' AND dst_cid=''' + @dst_cid + '''' 
    EXEC(@sql)
    SELECT @cnt = @cnt + @@ROWCOUNT

    FETCH NEXT FROM lnk_c INTO @name, @src_cid, @dst_cid
  END
  DEALLOCATE lnk_c
  SET @disp = 'No of non-existing links created: ' + @cnt + @disp

  /* Link Notification */
  INSERT #linkChanges(type, src_cid, src_id, dst_cid, dst_id, action_type)
    SELECT name, dst_cid, dst_id, src_cid, src_id, 'LinkCreated' FROM #linkCandidates
  EXEC cl_notifyLinkChanges

  -- Update _ParentCID and _ParentID for classes that have flag_directly_contained set
  -- Have to do this when name is a containment link and src_cid (child) is directly contained
  DELETE a FROM #linkCandidates a JOIN link_type lt ON (lt.link_type=a.name)
    WHERE lt.flag_containment=0

  DELETE a FROM #linkCandidates a JOIN obj_class oc ON (oc.cid=a.src_cid)
    WHERE oc.flag_direct_containment=0

  DECLARE @src_cname NVARCHAR(24)
  DECLARE lnk2_c CURSOR FOR SELECT DISTINCT src_cid FROM #linkCandidates
  OPEN lnk2_c
  FETCH NEXT FROM lnk2_c INTO @src_cid

  WHILE @@FETCH_STATUS <> -1 BEGIN
    SELECT @src_cname=cname FROM obj_class WHERE cid=@src_cid

    SET @sql = 'UPDATE v SET v._ParentCID=lc.dst_cid, v._ParentID=lc.dst_id ' + 
	       'FROM ' + @src_cname + '_V v JOIN #linkCandidates lc ON(lc.src_id=v.id) ' + 
	       'WHERE lc.src_cid=''' + @src_cid + ''''
    EXEC (@sql)

    FETCH NEXT FROM lnk2_c INTO @src_cid
  END
  DEALLOCATE lnk2_c
  TRUNCATE TABLE #linkCandidates
  TRUNCATE TABLE #linkChanges

  -- Finish up transaction
  IF @tcount = 0 
    COMMIT TRAN cl_processContd
